module AfterDo
  module Loader
    class AdvisedClass; end
    class AdvisedChild < AdvisedClass; end
    class OtherAdvisedChild < AdvisedClass; end

    class CoolAspect; end

    describe AspectApplier do
      subject(:applier) { described_class.new(aspect_config) }

      let(:aspect_config) { config.aspects.first }
      let(:config) do
        Config.new(fixture_path('aspect_config_with_subclass.yml'))
      end

      let(:aspect_instance) { CoolAspect.new }
      let(:aspect_class) { CoolAspect }
      let(:target_classes) { [AdvisedChild, OtherAdvisedChild] }

      describe '#apply' do
        subject(:apply) { applier.apply }

        before do
          allow(aspect_class).to receive(:new).and_return(aspect_instance)
          allow(aspect_instance).to receive(:add_callback)
        end

        it 'extends the target class with the AfterDo library' do
          target_classes.each do |target_class|
            expect(target_class).to receive(:extend).with(AfterDo)
          end

          apply
        end

        it 'creates an Aspect object of the correct class' do
          target_classes.each do |target_class|
            expect(aspect_class).to receive(:new).with(target_class)
          end

          apply
        end

        it 'applies the right advice to the target class' do
          expect(aspect_instance).to receive(:add_callback).with('initialize')
          expect(aspect_instance).to receive(:add_callback).with('merge')

          apply
        end

        context 'when applying goes wrong' do
          before do
            allow(AdvisedClass).to receive(:extend).and_raise(StandardError)
          end

          it { expect { apply }.to raise_error(Errors::CannotApplyAspect) }
        end
      end
    end
  end
end
