module AfterDo
  module Loader
    class AdvisedClass; end
    class CoolAspect; end
    class AdvisedChild < AdvisedClass; end
    class OtherAdvisedChild < AdvisedClass; end

    describe Config do
      subject(:config) { Config.new(config_path) }

      let(:config_path) { fixture_path('aspect_config.yml') }

      let(:config_hash) do
        {
          'aspects' => [
            {
              'name' => 'CoolAspect',
              'klass' => AfterDo::Loader::CoolAspect,
              'description' => "This aspect adds after_do callbacks to its \
target classes",
              'advices' => [
                {
                  'names' => %w(add_callback),
                  'targets' => [
                    {
                      'klass' => target_klasses,
                      'target_methods' => %w(initialize merge)
                    }
                  ]
                }
              ]
            }
          ]
        }
      end

      let(:config_os) do
        RecursiveOpenStruct.new(config_hash, recurse_over_arrays: true)
      end

      let(:target_klasses) { [AdvisedClass] }

      it { expect(config.aspects).to eq(config_os.aspects) }

      context 'when parsing goes wrong' do
        let(:config_path) { fixture_path('bad_aspect_config.yml') }

        it { expect { config }.to raise_error(Errors::InvalidConfig) }
      end

      context 'when advised class has subclasses' do
        let(:config_path) { fixture_path('aspect_config_with_subclass.yml') }

        let(:target_klasses) { [AdvisedChild, OtherAdvisedChild] }

        it { expect(config.aspects).to eq(config_os.aspects) }
      end
    end
  end
end
